@@ -2,22 +2,26 @@ import pickle
 
 import numpy as np
 from matplotlib import pyplot as plt
-from sklearn.cluster import KMeans, DBSCAN
+from sklearn.cluster import KMeans
 from scipy.spatial import cKDTree
 import logging
 
 from vizuka import vizualization
-from vizuka.clustering.clustering import Clusterizer
+from vizuka.clustering.clusterizer import Clusterizer
 
 
 class KmeansClusterizer(Clusterizer):
 
-    def __init__(self, n_clusters=120, *args, **kwargs):
+    required_arguments = ['Number of clusters']
+
+    def __init__(self, required_arguments={'Number of clusters':15}):
         """
         Uses sklearn kmeans, accepts same arguments.
         Default nb of cluster : 120
         """
-        self.engine = KMeans(n_clusters=n_clusters, *args, **kwargs)
+        self.engine = KMeans(
+                n_clusters=int(required_arguments['Number of clusters']),
+                )
         self.method='kmeans'
 
     def fit(self, xs):
